#include "osHandles.h"              // Includes all FreeRTOS Files

#include "System/cpu.h"             // Initialize CPU Hardware
#include "drivers/watchdog.h"		// Watchdog startup delay
#include "drivers/uart0.h"       	// Initialize UART
#include "drivers/uart1.h"
#include "utils/rprintf.h"			// Reduced printf.  STDIO uses lot of FLASH space & Stack space
#include "tasks/userInterface.hpp"	// User interface functions to interact through UART

typedef union
		{
			unsigned int full32bit;
			struct
			{
				char PO_0 : 2;
				char PO_1 : 2;
				char PO_2 : 2;
				char PO_3 : 2;
				char PO_4 : 2;
				char PO_5 : 2;
				char PO_6 : 2;
				char PO_7 : 2;
			};
		}__attribute__((packed)) MyPinSelType;

/*
void led_task (void *p)
{
	OSHANDLES *osHandles = (OSHANDLES*)p;
	MyPinSelType *pPinSel0 = (MyPinSelType *)PINSEL0;

	const int ledPinNum = 5;

	pPinSel0->PO_4 = 0;

	//PINSEL0 &= ~(0xF << 8);
	IODIR0 |= (1 << ledPinNum);
	IODIR0 &= ~(1 << 4);

	for(;;)
	{
		if(IOPIN0 & (1 << 4))
		{
			//IOPIN0 |= (1 << ledPinNum);
			IOSET0 = (1 << ledPinNum);
			vTaskDelay(250);
			IOCLR = (1 << ledPinNum);
			vTaskDelay(250);
		}
		else
		{
			//IOPIN0 &= ~(1 << ledPinNum);
			IOSET = (1 << ledPinNum);
		}

		const int delayInMs = 10;
		vTaskDelay(delayInMs);

	}

}*/

/* INTERRUPT VECTORS:
 * 0:    OS Timer Tick
 * 1:    Not Used
 * 2:    UART0 Interrupt
 * 3:    Not Used
 * 4:    Not Used
 * 5-16: Not Used
 */
int main (void)
{
	OSHANDLES SysHandles;        // Should contain all OS Handles
//	char buffer[512] = {0};

	cpuSetupHardware();          // Setup PLL, enable MAM etc.
	watchdogDelayUs(1000*1000);  // Some startup delay
	uart0Init(38400);
	uart1Init(9600);
//	ReceiveGPSData(buffer, 512);
//	rprintf("%s", buffer);

	rprintf_devopen(uart0PutChar);
	cpuPrintMemoryInfo();
	watchdogDelayUs(1000*1000);


	// Use the WATERMARK command to determine optimal Stack size of each task (set to high, then slowly decrease)
	// Priorities should be set according to response required
	if 	(pdPASS != xTaskCreate( uartUI, (signed char*)"Uart UI", STACK_BYTES(1024*6), &SysHandles, PRIORITY_LOW, &SysHandles.task.userInterface )|| (pdPASS!= xTaskCreate( mp3Task, (signed char*)"MP3", STACK_BYTES(1024*6), &SysHandles, PRIORITY_LOW, &SysHandles.task.mp3)))
	{
		rprintf("ERROR:  OUT OF MEMORY: Check OS Stack Size or task stack size.\n");
	}

	// Start FreeRTOS to begin servicing tasks created above, vTaskStartScheduler() will not "return"
	rprintf("\n-- Starting FreeRTOS --\n");
	vTaskStartScheduler();

	// In case OS is exited:
	rprintf("ERROR: Unexpected OS Exit!\n");

	return 0;
}

